/***************************************************************************
                          functions.cpp  -  description
                             -------------------
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "functions.h"
#include <kapp.h>
#include <qstringlist.h>
class QString;
class QComboBox;
class QWidget;
class QLineEdit;
class QLabel;
class QListBox;
class QRadioButton;


/* refresh *ALL* the data on the GUI (this is called on startup) */
int refreshGUIdata(QWidget *widget) {
  changing_text_from_signal = TRUE;
  /* refresh EAK types */
  refresh_kbtypes (widget);
  /* refresh CD-ROM entry */
  refresh_cdromdev (widget);
  /* refresh keyboard image */
  refresh_kbimage (widget);
  /* refresh key list */
  refresh_keylist (widget);
  /* refresh command/action */
  refresh_specials (widget);
  refresh_keycommand (widget, 0);
  changing_text_from_signal = FALSE;

  return (TRUE);
}

/* refresh EAK types */
/* only needed at startup */
int refresh_kbtypes (QWidget *widget) {
  QString tmp;
  QComboBox *guiwidget = static_cast<QComboBox*>((static_cast<KlineakConfig*>(widget))->getWidget("cbKeyboard"));
  for (int i=0; i<nrkeyboards; i++) {
   tmp = longkbname[i];
  	guiwidget->insertItem(tmp);
  }	
  guiwidget->setCurrentItem(myEAK.EAKnr);
  return (TRUE);
}
/* only needed at startup */
int refresh_cdromdev (QWidget *widget) {
  QString tmp = (char*)cdromdev;
  QLineEdit *guiwidget = static_cast<QLineEdit*>((static_cast<KlineakConfig*>(widget))->getWidget("txtCDRom"));;
  guiwidget->setText(tmp);
  return (TRUE);
}

/* refresh keyboard image */
int refresh_kbimage (QWidget *widget) {
  KStandardDirs *resources = KGlobal::dirs();
  static const QString pixmapdir = "klineakconfig/pixmaps/";
  QStringList out;
  QString pixmapname;

  if ( QString(myEAK.EAKtype) == "" )
    pixmapname="lineak_logo";
  else
    pixmapname=myEAK.EAKtype;
  pixmapname+=".xpm";

  resources->addPrefix(ROOTDIR);
//  out = resources->resourceDirs("data");
//  cout << "Looking for pixmaps in: ";
//  for (QStringList::Iterator it = out.begin(); it!=out.end();++it) {
//    	cout << *it << ":";
//  }
//  cout << endl;
  QLabel *kbimage  = static_cast<QLabel*>((static_cast<KlineakConfig*>(widget))->getWidget("pixPicture"));;
  QPixmap pixmap(resources->findResource("data", pixmapdir+pixmapname));
  kbimage->setPixmap(pixmap);
  return (TRUE);
}

/* refresh the keylist when the user selected some other keyboard */
int refresh_keylist (QWidget *widget) {
  QString tmp;
  QListBox *guiwidget = static_cast<QListBox*>((static_cast<KlineakConfig*>(widget))->getWidget("lstAvailableKeys"));
  EAkey *thiskey=NULL;
  guiwidget->clear();
  for (thiskey=myEAK.EAKeylist; thiskey!=NULL; thiskey=thiskey->next) {
	tmp = thiskey->EAkeyname;
	guiwidget->insertItem(tmp);
  }
  guiwidget->setSelected(0,true);
  selected_key = 0;
  return (TRUE);
}

/* refresh the command bound to a key when it's selected */
int refresh_keycommand (QWidget *widget, int keyindex) {
  QString tmp;
  QRadioButton *rbwidget;
  QLineEdit *guiwidget;
  QComboBox *cbwidget;
  EAkey *thiskey=myEAK.EAKeylist;
  changing_text_from_signal = TRUE;
  int i;
  for (i=0; i!=keyindex && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
	printf("invalid key index: %d\n",keyindex);
	return (FALSE);
  }
  if (is_special_action((char *)(thiskey->EAcommand))) { /* special action */
	rbwidget = static_cast<QRadioButton*>((static_cast<KlineakConfig*>(widget))->getWidget("radioRun"));
	rbwidget->setChecked(false);
	rbwidget = static_cast<QRadioButton*>((static_cast<KlineakConfig*>(widget))->getWidget("radioAction"));
	rbwidget->setChecked(true);
	guiwidget = static_cast<QLineEdit*>((static_cast<KlineakConfig*>(widget))->getWidget("txtCommand"));
	guiwidget->setText("");
	guiwidget->setEnabled(false);
	for (i=0; strcmp(specialacts[i].action,thiskey->EAcommand)!=0 && i<NR_SPECIALS; i++);
	cbwidget = static_cast<QComboBox*>((static_cast<KlineakConfig*>(widget))->getWidget("cbAction"));
	cbwidget->setCurrentText(QString(specialacts[i].longname));
  } else {                                                        /* system command */
	rbwidget = static_cast<QRadioButton*>((static_cast<KlineakConfig*>(widget))->getWidget("radioAction"));
	rbwidget->setChecked(false);
	rbwidget = static_cast<QRadioButton*>((static_cast<KlineakConfig*>(widget))->getWidget("radioRun"));
	rbwidget->setChecked(true);
	guiwidget = static_cast<QLineEdit*>((static_cast<KlineakConfig*>(widget))->getWidget("txtCommand"));
	tmp = (char *)(thiskey->EAcommand);
	guiwidget->setText(tmp);
	guiwidget->setReadOnly(false);
  }
  changing_text_from_signal = FALSE;
  return (TRUE);
}

/* refill special action menu (only needed on startup) */
int refresh_specials (QWidget *widget) {
  QString tmp;
  QComboBox *guiwidget = static_cast<QComboBox*>((static_cast<KlineakConfig*>(widget))->getWidget("cbAction"));
  for (int i=0; i<NR_SPECIALS; i++)
    add_specmenu_item(guiwidget, specialacts[i].action, specialacts[i].longname, TRUE);
  return (TRUE);
}

QWidget *add_specmenu_item (QWidget *menu, char *act, char *name, int add_callback) {
  QString tmp = name;
  QComboBox *item = static_cast<QComboBox*>(menu);
  item->insertItem(tmp);
  return item;
}

/* is a command a special action? */
int is_special_action (char *command) {
  if (strcmp(command, EAK_EJECT)==0 ||
	  strcmp(command, EAK_VOLUP)==0 ||
      strcmp(command, EAK_VOLDOWN)==0 ||
      strcmp(command, EAK_MUTE)==0 ||
      strcmp(command, EAK_SLEEP)==0)
    return (TRUE);
  else
	return (FALSE);
}

/* select a new keyboard */
void select_new_keyboard(QWidget *widget, gpointer data) {
  char *ndata = (char*)data;
  extern gboolean changing_text_from_signal;
  free(kbtype);
  kbtype=(char*)malloc(strlen(ndata)*sizeof(char));
  strcpy(kbtype, ndata);
  initEAK();
  changing_text_from_signal = TRUE;
  refresh_keylist (widget);
  refresh_keycommand (widget, 0); /* refresh_keylist selects 1st key, hence index 0 */
  refresh_kbimage (widget);
  changing_text_from_signal = FALSE;
  selected_key = 0;
}

/* change the key command data to the users input */
void change_key_command(EAkey *thiskey, gchar *entrytext) {
  free((void*)thiskey->EAcommand);
  thiskey->EAcommand = (char*)malloc((strlen(entrytext)+1)*sizeof(char));
  strcpy(thiskey->EAcommand,entrytext);
  strcat(thiskey->EAcommand,"\0");
}

/* wrapper for change_key_command, lookups the selected key */
void change_selectedkey_command(gchar *entrytext) {
  EAkey *thiskey;
  gint i;
  for (i=0, thiskey=myEAK.EAKeylist; i!=selected_key && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
    g_warning("invalid selected key index: %d\n",selected_key);
    g_free(entrytext);
    return;
  }
  if (strcmp(thiskey->EAcommand,(char *)entrytext) != 0)
	change_key_command(thiskey, entrytext);
}
