/***************************************************************************
                          klineakconfig.cpp  -  description
                             -------------------
    begin                : Tue Apr  9 18:33:44 EDT 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "klineakconfig.h"
#include "functions.h"

KlineakConfig::KlineakConfig(QWidget *parent, const char *name) : KlineakConfigUI(parent, name)
{
	ApplyUsed=false;
	  /* parse .conf and .def files, and init the EAK data */
  if (!parseconffile() || !parsedeffile())
	exit(0);
  if (!initEAK()) {
	cleanKeylist();
	exit(0);
  }
  refreshGUIdata(this);
}

KlineakConfig::~KlineakConfig() {
}

void KlineakConfig::updateAvailableKeys(int i) {
    if (strcmp(myEAK.EAKtype,(char *)cfgSectionNumberToName(i)) != 0) {
		select_new_keyboard(this,(gpointer)cfgSectionNumberToName(i));
  	}
}

void KlineakConfig::updateCDRom() {
  QString text = txtCDRom->text();
  if (text.compare(QString(cdromdev))) {
        free(cdromdev);
        cdromdev = (char*)malloc( (text.length()+1) *sizeof(char));
        strcpy(cdromdev,text.latin1());
        strcat(cdromdev,"\0");
  }
}
// Update the keylist mappings when a radio action is selected
void KlineakConfig::updateKeyList(int index) {
  if (!radioAction->isOn())
        return;

  QString text = QString(specialacts[index].action);
  char *tmpstr = (char*)malloc( (text.length()+1) *sizeof(char));
  strcpy(tmpstr,text.latin1());
  strcat(tmpstr,"\0");
  change_selectedkey_command(tmpstr);
}
// Update the keylist mappings when the text changes.
void KlineakConfig::updateKeyList(const QString& inText) {
  if (changing_text_from_signal)
        return;

  QString text = txtCommand->text();
  char *tmpstr = (char*)malloc( (text.length()+1) *sizeof(char));
  strcpy(tmpstr,text.latin1());
  strcat(tmpstr,"\0");
  change_selectedkey_command(tmpstr);
}

// When an item is selected in the keylist
void KlineakConfig::onKeyListSelect(QListBoxItem *box) {
  int index = lstAvailableKeys->currentItem();
  if (index != -1 ) {
  	changing_text_from_signal = TRUE;
    refresh_keycommand (this, index);
    changing_text_from_signal = FALSE;
    selected_key = lstAvailableKeys->currentItem();
  }
}
// When you click the OK button
void KlineakConfig::onOKButtonClicked() {
    onApplyButtonClicked();
	cleanexit();
}
// When you click the Apply button
bool KlineakConfig::onApplyButtonClicked() {
  gint thepid=0;
  /* get lineakd PID */
  if ((thepid = get_lineakd_pid())==0) {
        QMessageBox::information(this, "KLineakConfig",
        	"WARNING: Could not find lineakd running!\n"
         	"LinEAK configuration saved. You must start\n"
          "lineakd in order for your Easy Access Keys to work.\n"
          "At the Konsole prompt try typing: lineakd -b\n");
       if (!ApplyUsed) {
       	backup_conffile();
        	ApplyUsed=true;
       }
  		/* save the changes to a new conf file */
  		save_conffile();
       return false;
  }
  /* first time apply? -> backup conf file */
  if (!ApplyUsed) {
    ApplyUsed = true;
  	backup_conffile();
  }
  /* save the changes to a new conf file */
  save_conffile();
  /* rehash lineakd */
  kill(thepid, SIGHUP);
  return true;
}
// When the cancel button is clicked
void KlineakConfig::quit() {
  gint thepid=0;
  if (ApplyUsed) {
    if ((thepid = get_lineakd_pid())==0)
          	return;
        else {
          	restore_conffile();
      		kill(thepid, SIGHUP);
        }
  }
  cleanexit();
}
// Return a reference to a widget
QWidget* KlineakConfig::getWidget(QString name) {
	if (name == "cbKeyboard")
		return (static_cast<QWidget*>(cbKeyboard));
	if (name == "txtCDRom")
		return (txtCDRom);
	if (name == "pixPicture")
		return (pixPicture);
	if (name == "lstAvailableKeys")
		return (lstAvailableKeys);
	if (name == "radioRun")
		return (radioRun);
	if (name == "txtCommand")
		return (txtCommand);
	if (name == "radioAction")
		return (radioAction);
	if (name == "cbAction")
		return (cbAction);
	if (name == "btnCancel")
		return (btnCancel);
	if (name == "btnApply")
		return (btnApply);
	if (name == "btnOK")
		return (btnOK);	
}




