/***************************************************************************
                          support.c  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* global stuff */
#include "support.h"

/*
#include "callbacksproto.h"
 */
/* private stuff */
/*#include "supportpriv.h" */
#include "lineakconfig.h"



/* parse .conf file */
int parseconffile(void) {
  FILE *fp = NULL;
  char *homedir = NULL;

  if ( (homedir = (char *) getenv("HOME")) != NULL ) {
    cfile = malloc((strlen(homedir) + strlen(CONFFILE) + 1) * sizeof(char));
    strcpy(cfile, homedir);
    strcat(cfile, CONFFILE "\0");
    /* does the file exist? */
    if ((fp = fopen(cfile, "r")) != NULL) {
	  fclose(fp);
	  /* backup filename */
	  backupfile = malloc((strlen(homedir) + strlen(BAKFILE) + 1) * sizeof(char));
      strcpy(backupfile, homedir);
      strcat(backupfile, BAKFILE "\0");
	  /* parse the contents of the conf file */
	  if ((cfgParse(cfile, confdata, CFG_SIMPLE)) == -1) {
	    /* error parsing file */
	    printf("FATAL ERROR occurred while reading configuration data!\n");
	    return (FALSE);
	  }
    } else {
	  /* file does not exist */
      printf("FATAL ERROR: configuration file %s not found!\n             use 'lineakd -c TYPE' to create one\n",cfile);
  	  return (FALSE);
    }

  } else {
    printf("ERROR: no $HOME environment setting found\n");
    return (FALSE);
  }

  return (TRUE);
}

/* parse keyboard definition file */
int parsedeffile(void) {
  FILE *fp = NULL;
  /* SET full path of definition file ($sysconfdir/lineakkb.def) */
  dfile = malloc((strlen(CONFDIR) + 1 + strlen(DEFFILE) + 1)*sizeof(char));
  strcpy(dfile, CONFDIR);
  strcat(dfile, "/");
  strcat(dfile, DEFFILE);
  strcat(dfile, "\0");

  /* does the file exist? */
  if ((fp = fopen(dfile, "r")) != NULL) {
    fclose(fp);
	/* it exists, parse the contents */
	if ((nrkeyboards = cfgParse(dfile, defdata, CFG_INI)) == -1) {
	  /* error parsing file */
	  printf("FATAL ERROR occurred while reading definition data!\n");
  	  return(FALSE);
	}
  } else {
	/* file does not exist */
    printf("FATAL ERROR: definition file %s not found!\n",dfile);
	return(FALSE);
  }

  return (TRUE);
}

/* init the EAKeyboard */
int initEAK (void) {
  cfgList *l = NULL;
  EAkey *tmp = NULL;
  int keycount = 0;
  cleanKeylist();
  myEAK.EAKnr = cfgSectionNameToNumber(kbtype);
  myEAK.EAKtype = kbtype;
/*  printf("kytype is %s\n",kbtype); */
  myEAK.EAKname = longkbname[myEAK.EAKnr];
  if (keylist[myEAK.EAKnr] != NULL) {
	myEAK.EAKeylist = tmp = (EAkey *) malloc(sizeof(EAkey));
	for (l=keylist[myEAK.EAKnr]; l!=NULL; l=l->next) {
      keycount++;
	  tmp->EAkeyname = l->str;

	  tmp->EAcommand = NULL;
	  if (fetchVarFromCfgFile(cfile, tmp->EAkeyname, &(tmp->EAcommand), CFG_STRING, CFG_SIMPLE, 0, NULL) == -1) {
		free(tmp->EAcommand);
		tmp->EAcommand = malloc(1*sizeof(char));
	    strcpy(tmp->EAcommand,"\0");
	  }
	  if (l->next != NULL) {
		tmp->next = (EAkey *) malloc(sizeof(EAkey));
		tmp = tmp->next;
	  } else {
		tmp->next = NULL;
	  }
	}
  } else {
	myEAK.EAKeylist = NULL;
  }
  return (TRUE);
}

/* destroy the EAkeys linked list */
void cleanKeylist (void) {
  EAkey *tmp = NULL, *tmp2 = NULL;
  tmp = myEAK.EAKeylist;
  while (tmp!=NULL) {
	tmp2 = tmp->next;
	free(tmp->EAcommand);
	free(tmp);
	tmp = tmp2;
  }
}

/* cleanup */
gint cleanexit() {
  cleanKeylist();
  exit(1);
  return FALSE;
}

/* backup the conf file so we can cancel the changes */
gboolean backup_conffile (void) {
  FILE *orig, *backup;
  int chr;
  if ((orig=fopen(cfile,"r"))==NULL) {
	g_warning("error opening configuration file %s for read\n",cfile);
	return FALSE;
  }
  if ((backup=fopen(backupfile, "w"))==NULL) {
	g_warning("error opening backup configuration file %s for write\n",backupfile);
	fclose(orig);
	return FALSE;
  }
  while ((chr=fgetc(orig))!=EOF) fputc(chr,backup);
  fclose(orig);
  fclose(backup);
  return TRUE;
}

/* restore the conf file from backup */
gboolean restore_conffile (void) {
  FILE *orig, *dest;
  int chr;
  if ((orig=fopen(backupfile,"r"))==NULL) {
	g_warning("error opening backup configuration file %s for read\n",backupfile);
	return FALSE;
  }
  if ((dest=fopen(cfile, "w"))==NULL) {
	g_warning("error opening configuration file %s for write\n",cfile);
	fclose(orig);
	return FALSE;
  }
  while ((chr=fgetc(orig))!=EOF) fputc(chr,dest);
  fclose(orig);
  fclose(dest);
  return TRUE;
}

/* save the settings to the conf file */
gboolean save_conffile (void) {
  FILE *dest;
  EAkey *key;
  if ((dest=fopen(cfile, "w"))==NULL) {
	g_warning("error opening configuration file %s for write\n",cfile);
	return FALSE;
  }
  fprintf(dest,"%s# LinEAK Configuration for %s\n# (generated by lineakconfig)\n\n# Global settings\n"
               "KeyboardType\t\t= %s\n"
               "CdromDevice\t\t= %s\n"
               "\n# Specific keys of your keyboard\n", CONF_HEADER, myEAK.EAKname, kbtype, cdromdev);
  key = myEAK.EAKeylist;
  while (key!=NULL) {
	fprintf(dest,"%s%s= \"%s\"\n",key->EAkeyname,strlen(key->EAkeyname)>7?"\t":"\t\t",key->EAcommand);
	key = key->next;
  }
  fprintf(dest,"\n# end lineakd.conf\n");
  fclose(dest);
  return TRUE;
}

gint get_lineakd_pid(void) {
  FILE *fp;
  char *home, *pidfile;
  gint thepid=0;
  home = getenv("HOME");
  pidfile = malloc( (strlen(home)+strlen(PIDFILE)+1) *sizeof(char));
  sprintf(pidfile,"%s%s",home,PIDFILE);
  if ((fp = fopen(pidfile,"r")) != NULL) {
	fscanf(fp, "%d", (int *)&thepid);
	fclose(fp);
	free(home);
	free(pidfile);
  } else {
	free(home);
	free(pidfile);
  }
  return thepid;
}


/**********************************************************************************************/
